@@ -22,7 +22,7 @@
 	This file incorporates work covered by the following copyright and
 	permission notice:
 
-	Copyright (c) 2013-2021 Cong Xu
+	Copyright (c) 2013-2024 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -88,6 +88,7 @@
 #define FLAGS_SLEEPALWAYS (1 << 24)
 #define FLAGS_AWAKEALWAYS (1 << 25)
 #define FLAGS_RESCUED (1 << 26) // Run towards exit
+#define FLAGS_MOVE_AND_SHOOT (1 << 27)	// Can move and shoot at the same time
 
 typedef enum
 {
@@ -133,8 +134,7 @@ typedef struct Actor
 	Weapon guns[MAX_WEAPONS];
 	CArray ammo; // of int
 	int gunIndex;
-	// TODO: multiple grenade slots?
-	int grenadeIndex;
+	int lastGunIdx;
 
 	int health;
 	// A counter for player death
@@ -172,6 +172,8 @@ typedef struct Actor
 	color_t footprintMask;
 	int footprintCounter;
 
+	bool hasShot;
+
 	// Signals to other AIs what this actor is doing
 	ActorAction action;
 	AIContext *aiContext;
@@ -191,13 +193,12 @@ extern CArray gActors; // of TActor
 
 void ActorSetState(TActor *actor, const ActorAnimation state);
 void UpdateActorState(TActor *actor, int ticks);
-bool TryMoveActor(TActor *actor, struct vec2 pos);
 void ActorMove(const NActorMove am);
 void CommandActor(TActor *actor, int cmd, int ticks);
 void SlideActor(TActor *actor, int cmd);
 void UpdateAllActors(const int ticks);
 void ActorsPilotVehicles(void);
-void ActorHeal(TActor *actor, int health);
+void ActorHeal(TActor *actor, const int amount, const bool exceedMax);
 void InjureActor(TActor *actor, int injury);
 
 void ActorAddAmmo(TActor *actor, const int ammoId, const int amount);
@@ -217,7 +218,7 @@ void ActorDestroy(TActor *a);
 TActor *ActorGetByUID(const int uid);
 const Character *ActorGetCharacter(const TActor *a);
 #define ACTOR_GET_GUN(a) (&(a)->guns[(a)->gunIndex])
-#define ACTOR_GET_GRENADE(a) (&(a)->guns[(a)->grenadeIndex + MAX_GUNS])
+#define ACTOR_GET_GRENADE(a) (&(a)->guns[MAX_GUNS])
 #define ACTOR_GET_WEAPON(a)                                                   \
 	(ACTOR_GET_GUN(a)->Gun != NULL ? ACTOR_GET_GUN(a) : ACTOR_GET_GRENADE(a))
 struct vec2 ActorGetAverageWeaponMuzzleOffset(const TActor *a);
@@ -228,13 +229,15 @@ int ActorWeaponGetAmmo(
 	const TActor *a, const WeaponClass *wc, const int barrel);
 // Returns -1 if no barrels can fire
 int ActorGetCanFireBarrel(const TActor *a, const Weapon *w);
-bool ActorTrySwitchWeapon(const TActor *a, const bool allGuns);
+bool ActorCanSwitchToWeapon(const TActor *a, const int slot);
+bool ActorTrySwitchWeapon(const TActor *a);
 // Returns -1 if actor doesn't have gun
 int ActorFindGun(const TActor *a, const WeaponClass *wc);
 int ActorGetNumWeapons(const TActor *a);
 int ActorGetNumGuns(const TActor *a);
 int ActorGetNumGrenades(const TActor *a);
 void ActorSwitchGun(const NActorSwitchGun sg);
+void ActorPickupGun(const TActor *a, const WeaponClass *wc);
 bool ActorIsImmune(const TActor *actor, const special_damage_e damage);
 bool ActorTakesDamage(const TActor *actor, const special_damage_e damage);
 void ActorHit(const NThingDamage d);
@@ -242,8 +245,10 @@ bool ActorIsInvulnerable(
 	const TActor *a, const int flags, const int playerUID, const GameMode mode,
 	const special_damage_e special);
 
+int ActorGetMaxHeal(const TActor *a, const bool exceedMax);
 int ActorGetHealthPercent(const TActor *a);
 bool ActorIsLowHealth(const TActor *a);
 bool ActorIsGrimacing(const TActor *a);
 
 bool ActorIsLocalPlayer(const int uid);
+void ActorPersistPlayerWeaponsAndAmmo(const TActor *a);
