@@ -1,4 +1,5 @@
-#pragma once 
+#pragma once
+#include <stdbool.h>
 #include <stdint.h>
 
 #define BYTE uint8_t
@@ -7,4 +8,5 @@
 
 void ExpandCarmack(const unsigned char *in, unsigned char *out);
 void ExpandRLEW(
-	const unsigned char *in, unsigned char *out, const WORD rlewTag);
+	const unsigned char *in, unsigned char *out, const WORD rlewTag,
+	const bool hasFinalLength);
