@@ -22,7 +22,7 @@
 	This file incorporates work covered by the following copyright and
 	permission notice:
 
-	Copyright (c) 2013-2016, 2018-2021 Cong Xu
+	Copyright (c) 2013-2016, 2018-2021, 2023 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -286,6 +286,7 @@ void ConvertCharacterColors(
 	c->Legs = ColorTint(PaletteToColor(cShadePalettes[leg]), tint);
 	c->Hair = ColorTint(PaletteToColor(cShadePalettes[hair]), tint);
 	c->Feet = c->Legs;
+	c->Facehair = c->Hat = c->Glasses = c->Hair;
 }
 
 // Hair colour correction; some characters had no hair but now with
@@ -344,7 +345,7 @@ void ConvertCharacter(Character *c, TBadGuy *b)
 {
 	const char *face = IntCharacterFace(b->facePic);
 	char *newFace = NULL;
-	CharacterOldFaceToHair(face, &newFace, &c->Hair);
+	CharacterOldFaceToHeadParts(face, &newFace, c->HeadParts);
 	c->Class = StrCharacterClass(newFace);
 	CFREE(newFace);
 	c->speed = b->speed / 256.0f;
@@ -358,6 +359,7 @@ void ConvertCharacter(Character *c, TBadGuy *b)
 		&c->Colors);
 	ConvertHairColors(c, face);
 	c->maxHealth = b->health;
+	c->excessHealth = c->maxHealth * 2;
 	c->flags = b->flags;
 }
 static void ConvertObjective(Objective *dest, struct MissionObjectiveOld *src)
@@ -372,10 +374,10 @@ static void ConvertObjective(Objective *dest, struct MissionObjectiveOld *src)
 	switch (dest->Type)
 	{
 	case OBJECTIVE_COLLECT:
-		dest->u.Pickup = IntPickupClass(dest->u.Index);
+		ObjectiveSetPickup(dest, IntPickupClass(dest->u.Index));
 		break;
 	case OBJECTIVE_DESTROY:
-		dest->u.MapObject = IntMapObject(dest->u.Index);
+		ObjectiveSetDestroy(dest, IntMapObject(dest->u.Index));
 		break;
 	default:
 		// do nothing
@@ -602,18 +604,24 @@ const char *GetHomeDirectory(void)
 
 /* GetConfigFilePath()
  *
- * returns a full path to a data file...
+ * returns a full path to a config file
  */
 char cfpath[CDOGS_PATH_MAX];
 const char *GetConfigFilePath(const char *name)
 {
-	const char *homedir = GetHomeDirectory();
-
-	strcpy(cfpath, homedir);
-
+	const char *xdgConfigDir = getenv("XDG_CONFIG_HOME");
+	if (xdgConfigDir != NULL)
+	{
+		sprintf(cfpath, "%s/cdogs-sdl/", xdgConfigDir);
+	}
+	else
+	{
+		const char *homedir = GetHomeDirectory();
+		strcpy(cfpath, homedir);
 #ifndef __EMSCRIPTEN__
-	strcat(cfpath, CDOGS_CFG_DIR);
+		strcat(cfpath, CDOGS_CFG_DIR);
 #endif
+	}
 	strcat(cfpath, name);
 
 	return cfpath;
