@@ -1,7 +1,7 @@
 /*
 	C-Dogs SDL
 	A port of the legendary (and fun) action/arcade cdogs.
-	Copyright (c) 2013-2017, 2019-2021 Cong Xu
+	Copyright (c) 2013-2017, 2019-2021, 2024 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -40,6 +40,8 @@
 #define GUN_ICON_WIDTH 14
 #define PLAYER_ICON_WIDTH 20
 #define BAR_HEIGHT 13
+#define GUN_CHANGE_FLASH_MS 500
+#define FLASH_PERIOD_MS 100
 
 void HUDPlayerInit(HUDPlayer *h)
 {
@@ -73,6 +75,37 @@ void HUDPlayerUpdate(HUDPlayer *h, const PlayerData *p, const int ms)
 
 	// Health
 	HealthGaugeUpdate(&h->healthGauge, a, ms);
+
+	// Initialise guns
+	const Weapon *gun = ACTOR_GET_GUN(a);
+	const WeaponClass *gunWc = gun->Gun;
+	if (h->gunWc == NULL)
+	{
+		h->gunWc = gunWc;
+	}
+	h->gunChangeMs = MAX(0, h->gunChangeMs - ms);
+	if (h->gunWc != gunWc)
+	{
+		// Gun has changed
+		h->gunWc = gunWc;
+		h->gunChangeMs = GUN_CHANGE_FLASH_MS;
+	}
+	const Weapon *grenade = ACTOR_GET_GRENADE(a);
+	const WeaponClass *grenadeWc = grenade->Gun;
+	const int grenadeAmount = ActorWeaponGetAmmo(a, grenadeWc, 0);
+	if (h->grenadeWc == NULL)
+	{
+		h->grenadeWc = grenadeWc;
+		h->grenadeAmount = grenadeAmount;
+	}
+	h->grenadeChangeMs = MAX(0, h->grenadeChangeMs - ms);
+	if (h->grenadeWc != grenadeWc || h->grenadeAmount != grenadeAmount)
+	{
+		// Grenade has changed
+		h->grenadeWc = grenadeWc;
+		h->grenadeAmount = grenadeAmount;
+		h->grenadeChangeMs = GUN_CHANGE_FLASH_MS;
+	}
 }
 
 static void DrawPlayerStatus(
@@ -105,9 +138,11 @@ static void DrawWeaponStatus(
 	GraphicsDevice *g, const PicManager *pm, const TActor *actor,
 	const int flags, const Rect2i r, const color_t mask);
 static void DrawGunIcons(
-	GraphicsDevice *g, const TActor *actor, const int flags, const Rect2i r);
+	GraphicsDevice *g, const TActor *actor, const int flags,
+	const HUDPlayer *h, const Rect2i r);
 static void DrawGrenadeStatus(
-	GraphicsDevice *g, const TActor *a, const int flags, const Rect2i r);
+	GraphicsDevice *g, const TActor *a, const int flags, const HUDPlayer *h,
+	const Rect2i r);
 static void DrawRadar(
 	GraphicsDevice *device, const TActor *p, const int flags,
 	const bool showExit);
@@ -160,9 +195,9 @@ static void DrawPlayerStatus(
 	opts.Area = gGraphicsDevice.cachedConfig.Res;
 
 	DrawScore(hud->device, &gPicManager, p, data->Stats.Score, flags, r, mask);
-	DrawGrenadeStatus(hud->device, p, flags, r);
+	DrawGrenadeStatus(hud->device, p, flags, h, r);
 	DrawWeaponStatus(hud->device, &gPicManager, p, flags, r, mask);
-	DrawGunIcons(hud->device, p, flags, r);
+	DrawGunIcons(hud->device, p, flags, h, r);
 	DrawLives(hud->device, data, opts.HAlign, opts.VAlign);
 	DrawHealth(hud->device, p, flags, h, r, mask);
 
@@ -356,8 +391,7 @@ static void DrawWeaponStatus(
 			const Pic *fillPic = PicManagerGetPic(pm, "hud/gauge_small_inner");
 			const int ammoMax = ammo->Max ? ammo->Max : amount;
 			const struct vec2i fillPicSize = svec2i(
-				MAX(1, (AMMO_WIDTH - 1) * amount / ammoMax),
-				fillPic->size.y);
+				MAX(1, (AMMO_WIDTH - 1) * amount / ammoMax), fillPic->size.y);
 			Draw9Slice(
 				g, fillPic, Rect2iNew(svec2i(pos.x, pos.y), fillPicSize), 1, 1,
 				1, 1, false, colorBlue, SDL_FLIP_NONE);
@@ -411,7 +445,8 @@ static void DrawWeaponStatus(
 }
 
 static void DrawGunIcons(
-	GraphicsDevice *g, const TActor *actor, const int flags, const Rect2i r)
+	GraphicsDevice *g, const TActor *actor, const int flags,
+	const HUDPlayer *h, const Rect2i r)
 {
 	if (actor == NULL)
 	{
@@ -449,16 +484,22 @@ static void DrawGunIcons(
 	}
 
 	// Gun icon
+	const color_t gunIconMask =
+		h->gunChangeMs > 0 &&
+				(h->gunChangeMs % FLASH_PERIOD_MS) < FLASH_PERIOD_MS / 2
+			? colorDarkGray
+			: colorWhite;
 	PicRender(
-		wc->Icon, g->gameWindow.renderer, pos, colorWhite, 0.0, svec2_one(),
+		wc->Icon, g->gameWindow.renderer, pos, gunIconMask, 0.0, svec2_one(),
 		SDL_FLIP_NONE, Rect2iZero());
 }
 
 static void DrawGrenadeIcons(
-	GraphicsDevice *g, const Pic *icon, const struct vec2i pos,
-	const int width, const int amount);
+	GraphicsDevice *g, const HUDPlayer *h, const Pic *icon,
+	const struct vec2i pos, const int width, const int amount);
 static void DrawGrenadeStatus(
-	GraphicsDevice *g, const TActor *a, const int flags, const Rect2i r)
+	GraphicsDevice *g, const TActor *a, const int flags, const HUDPlayer *h,
+	const Rect2i r)
 {
 	if (a == NULL)
 	{
@@ -492,11 +533,11 @@ static void DrawGrenadeStatus(
 	const Pic *icon = wc->Icon;
 	if (useAmmo && amount > 0 && amount <= MAX_GRENADE_ICONS)
 	{
-		DrawGrenadeIcons(g, icon, pos, GRENADES_WIDTH, amount);
+		DrawGrenadeIcons(g, h, icon, pos, GRENADES_WIDTH, amount);
 	}
 	else
 	{
-		DrawGrenadeIcons(g, icon, pos, GRENADES_WIDTH, 1);
+		DrawGrenadeIcons(g, h, icon, pos, GRENADES_WIDTH, 1);
 		char buf[256];
 		if (amount >= 0)
 		{
@@ -512,15 +553,20 @@ static void DrawGrenadeStatus(
 	}
 }
 static void DrawGrenadeIcons(
-	GraphicsDevice *g, const Pic *icon, const struct vec2i pos,
-	const int width, const int amount)
+	GraphicsDevice *g, const HUDPlayer *h, const Pic *icon,
+	const struct vec2i pos, const int width, const int amount)
 {
 	const int dx = width / MAX_GRENADE_ICONS;
+	const color_t grenadeIconMask =
+		h->grenadeChangeMs > 0 &&
+				(h->grenadeChangeMs % FLASH_PERIOD_MS) < FLASH_PERIOD_MS / 2
+			? colorDarkGray
+			: colorWhite;
 	for (int i = 0; i < amount; i++)
 	{
 		const struct vec2i drawPos = svec2i(pos.x + i * dx, pos.y);
 		PicRender(
-			icon, g->gameWindow.renderer, drawPos, colorWhite, 0.0,
+			icon, g->gameWindow.renderer, drawPos, grenadeIconMask, 0.0,
 			svec2_one(), SDL_FLIP_NONE, Rect2iZero());
 	}
 }
@@ -660,8 +706,15 @@ static void DrawPlayerObjectiveCompass(
 	{
 		return;
 	}
-	// deviding the screen by 4 and 2, which draws the arrows half way from the screen edge to be centered
-	Rect2i r = Rect2iNew(svec2i(hud->device->cachedConfig.Res.x / 4, hud->device->cachedConfig.Res.y / 4), svec2i(hud->device->cachedConfig.Res.x / 2, hud->device->cachedConfig.Res.y / 2));
+	// deviding the screen by 4 and 2, which draws the arrows half way from the
+	// screen edge to be centered
+	Rect2i r = Rect2iNew(
+		svec2i(
+			hud->device->cachedConfig.Res.x / 4,
+			hud->device->cachedConfig.Res.y / 4),
+		svec2i(
+			hud->device->cachedConfig.Res.x / 2,
+			hud->device->cachedConfig.Res.y / 2));
 	if (hudPlayerIndex & 1)
 	{
 		r.Pos.x = r.Size.x;
@@ -748,9 +801,9 @@ static void DrawCompassArrow(
 	const struct vec2 compassV = svec2_subtract(
 		pos,
 		// Offset a little so the arrow points exactly at center of objective
-		svec2(playerPos.x - TILE_WIDTH / 2, playerPos.y - 20));
+		svec2(playerPos.x - TILE_WIDTH / 2, playerPos.y));
 	// Don't draw if objective is on screen
-	if (fabsf(compassV.x) < r.Size.x / 2 && fabsf(compassV.y) < r.Size.y / 2)
+	if (fabsf(compassV.x) < r.Size.x && fabsf(compassV.y) < r.Size.y)
 	{
 		return;
 	}
