@@ -1,7 +1,7 @@
 /*
 	C-Dogs SDL
 	A port of the legendary (and fun) action/arcade cdogs.
-	Copyright (c) 2014-2021 Cong Xu
+	Copyright (c) 2014-2023 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -80,7 +80,7 @@ int MapLoadCampaignJSON(const char *filename, CampaignSetting *c, int *version)
 		}
 	}
 	MapNewLoadCampaignJSON(root, c);
-	
+
 bail:
 	json_free_value(&root);
 	return err;
@@ -227,7 +227,9 @@ static void LoadArchivePics(PicManager *pm, map_t cc, const char *archive)
 {
 	char path[CDOGS_PATH_MAX];
 	sprintf(path, "%s/graphics", archive);
-	PicManagerLoadDir(pm, path, NULL, pm->customPics, pm->customSprites);
+	PicManagerLoadDir(pm, path, NULL, pm->customPics, pm->customSprites, false);
+	sprintf(path, "%s/graphics_hd", archive);
+	PicManagerLoadDir(pm, path, NULL, pm->customPics, pm->customSprites, true);
 	CharSpriteClassesLoadDir(cc, archive);
 }
 
@@ -304,8 +306,14 @@ int MapArchiveSave(const char *filename, CampaignSetting *c)
 	AddStringPair(root, "Description", c->Description);
 	AddBoolPair(root, "Ammo", c->Ammo);
 	AddBoolPair(root, "SkipWeaponMenu", c->SkipWeaponMenu);
+	AddBoolPair(root, "BuyAndSell", c->BuyAndSell);
 	AddBoolPair(root, "RandomPickups", c->RandomPickups);
 	AddIntPair(root, "DoorOpenTicks", c->DoorOpenTicks);
+	AddIntPair(root, "Lives", c->Lives);
+	AddIntPair(root, "MaxLives", c->MaxLives);
+	AddIntPair(root, "PlayerHP", c->PlayerHP);
+	AddIntPair(root, "PlayerMaxHP", c->PlayerMaxHP);
+	AddIntPair(root, "PlayerExcessHP", c->PlayerExcessHP);
 	AddIntPair(root, "Missions", (int)c->Missions.size);
 	char buf2[CDOGS_PATH_MAX];
 	sprintf(buf2, "%s/campaign.json", buf);
@@ -373,6 +381,17 @@ static json_t *SaveMissions(CArray *a)
 			json_insert_child(modsNode, modNode);
 		}
 		json_insert_pair_into_object(node, "MapObjectDensities", modsNode);
+		json_t *pcsNode = json_new_array();
+		for (int j = 0; j < (int)mission->PickupCounts.size; j++)
+		{
+			const PickupCount *pc =
+				CArrayGet(&mission->PickupCounts, j);
+			json_t *pcNode = json_new_object();
+			AddStringPair(pcNode, "Pickup", pc->P->Name);
+			AddIntPair(pcNode, "Count", pc->Count);
+			json_insert_child(pcsNode, pcNode);
+		}
+		json_insert_pair_into_object(node, "PickupCounts", pcsNode);
 
 		AddIntPair(node, "EnemyDensity", mission->EnemyDensity);
 		json_insert_pair_into_object(
@@ -473,14 +492,11 @@ static json_t *SaveWeapons(const CArray *weapons)
 static json_t *SaveMissionTileClasses(const MissionTileClasses *mtc)
 {
 	json_t *node = json_new_object();
-	json_insert_pair_into_object(
-		node, "Wall", TileClassSaveJSON(&mtc->Wall));
+	json_insert_pair_into_object(node, "Wall", TileClassSaveJSON(&mtc->Wall));
 	json_insert_pair_into_object(
 		node, "Floor", TileClassSaveJSON(&mtc->Floor));
-	json_insert_pair_into_object(
-		node, "Room", TileClassSaveJSON(&mtc->Room));
-	json_insert_pair_into_object(
-		node, "Door", TileClassSaveJSON(&mtc->Door));
+	json_insert_pair_into_object(node, "Room", TileClassSaveJSON(&mtc->Room));
+	json_insert_pair_into_object(node, "Door", TileClassSaveJSON(&mtc->Door));
 	return node;
 }
 static json_t *SavePillars(const PillarParams p)
@@ -501,6 +517,22 @@ static json_t *SaveDoors(const DoorParams d)
 	return node;
 }
 
+static json_t *SavePickupObjectives(const Objective *o)
+{
+	json_t *node = json_new_array();
+	CA_FOREACH(const PickupClass *, p, o->u.Pickups)
+	json_insert_child(node, json_new_string((*p)->Name));
+	CA_FOREACH_END()
+	return node;
+}
+static json_t *SaveDestroyObjectives(const Objective *o)
+{
+	json_t *node = json_new_array();
+	CA_FOREACH(const MapObject *, mo, o->u.MapObjects)
+	json_insert_child(node, json_new_string((*mo)->Name));
+	CA_FOREACH_END()
+	return node;
+}
 static json_t *SaveObjectives(CArray *a)
 {
 	json_t *objectivesNode = json_new_array();
@@ -511,10 +543,12 @@ static json_t *SaveObjectives(CArray *a)
 	switch (o->Type)
 	{
 	case OBJECTIVE_COLLECT:
-		AddStringPair(objNode, "Pickup", o->u.Pickup->Name);
+		json_insert_pair_into_object(
+			objNode, "Pickups", SavePickupObjectives(o));
 		break;
 	case OBJECTIVE_DESTROY:
-		AddStringPair(objNode, "MapObject", o->u.MapObject->Name);
+		json_insert_pair_into_object(
+			objNode, "MapObjects", SaveDestroyObjectives(o));
 		break;
 	default:
 		AddIntPair(objNode, "Index", o->u.Index);
