@@ -1,7 +1,7 @@
 /*
 	C-Dogs SDL
 	A port of the legendary (and fun) action/arcade cdogs.
-	Copyright (c) 2020-2023 Cong Xu
+	Copyright (c) 2020-2024 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -41,9 +41,11 @@ CWolfMap *defaultSpearMap = NULL;
 
 #define WOLF_STEAM_NAME "Wolfenstein 3D"
 #define SPEAR_STEAM_NAME "Spear of Destiny"
+#define N3D_STEAM_NAME "Super 3-D Noah's Ark"
 #define WOLF_GOG_ID "1441705046"
 #define WOLF_SPEAR_GOG_ID "1441705226"
 #define SPEAR_GOG_ID "1441705126"
+#define N3D_GOG_ID "1672565562"
 #define WOLF_DATA_DIR "data/.wolf3d/"
 
 #define TILE_CLASS_WALL_OFFSET 63
@@ -95,8 +97,13 @@ static void GetCampaignPath(
 			break;
 		}
 		break;
+	case CWMAPTYPE_N3D:
+		GetDataFilePath(buf, WOLF_DATA_DIR "N3D.cdogscpn");
+		break;
 	default:
-		CASSERT(false, "unknown map type");
+		// TODO: implement unknown map types
+		LOG(LM_MAP, LL_ERROR, "Unknown wolf map type %d", (int)type);
+		buf[0] = '\0';
 		break;
 	}
 }
@@ -155,7 +162,21 @@ static const char *soundsSOD[] = {
 	"chars/die/trans", "chars/alert/bill", "chars/die/bill",
 	"chars/die/ubermutant", "chars/alert/knight", "chars/die/knight",
 	"chars/alert/angel", "chars/die/angel", "chaingun_pickup", "spear"};
-// TODO BS6:
+static const char *soundsN3D[] = {
+	// 0-9
+	"chars/alert/antelope", "chars/alert/bear", "bullet_pickup",
+	"chars/alert/camel", "door_close", "cantaloupe", "cantaloupe_feeder",
+	"goat_kick", "gulp?", "chars/die/animal",
+	// 10-19
+	"chars/alert/elephant", "1up", "super_feeder", "chars/alert/giraffe",
+	"chars/alert/goat", "small_feeder", "doof?", "chars/alert/kangaroo",
+	"whistle", "hand_feed",
+	// 20-29
+	"chars/alert/monkey", "footsteps/bear", "door", "chars/alert/ostrich",
+	"chars/alert/ox", "hurt", "hahaha", "secret_door", "chars/alert/sheep",
+	"large_feeder",
+	// 30-32
+	"spit", "watermelon", "watermelon_feeder"}; // TODO BS6:
 // https://github.com/bibendovsky/bstone/blob/3dea1ef72a101519afd17aa95c881da40a18040d/src/bstone_audio_content_mgr.cpp#L320-L398
 static const char *GetSound(const CWMapType type, const int i)
 {
@@ -168,6 +189,9 @@ static const char *GetSound(const CWMapType type, const int i)
 		return soundsW6[i];
 	case CWMAPTYPE_SOD:
 		return soundsSOD[i];
+	case CWMAPTYPE_N3D:
+		return soundsN3D[i];
+		break;
 	default:
 		CASSERT(false, "unknown map type");
 		return NULL;
@@ -428,6 +452,9 @@ static const char *GetAdlibSound(const CWMapType type, const int i)
 		return adlibSoundsW6[i];
 	case CWMAPTYPE_SOD:
 		return adlibSoundsSOD[i];
+	case CWMAPTYPE_N3D:
+		// Not using any adlib sounds
+		return NULL;
 	default:
 		CASSERT(false, "unknown map type");
 		return NULL;
@@ -442,19 +469,23 @@ static const CWSongType songsCampaign[] = {
 	SONG_ROSTER,  // lose
 	SONG_VICTORY, // victory
 };
-static Mix_Chunk *LoadMusic(const CWolfMap *map, const int i)
+static bool LoadMusic(CWolfMap *map, MusicChunk *chunk, const int i)
 {
 	char *data;
 	size_t len;
-	const int err = CWAudioGetMusic(&map->audio, i, &data, &len);
+	const int err = CWAudioGetMusic(&map->audio, map->type, i, &data, &len);
 	if (err != 0)
 	{
-		goto bail;
+		return false;
 	}
-	return Mix_QuickLoad_RAW((Uint8 *)data, (Uint32)len);
-
-bail:
-	return NULL;
+	if (map->type == CWMAPTYPE_N3D)
+	{
+		SDL_RWops *rwops = SDL_RWFromMem(data, (int)len);
+		chunk->u.Music = Mix_LoadMUS_RW(rwops, 1);
+		return true;
+	}
+	chunk->u.Chunk = Mix_QuickLoad_RAW((Uint8 *)data, (Uint32)len);
+	return false;
 }
 
 static bool IsDefaultMap(const char *filename)
@@ -471,6 +502,8 @@ static bool IsDefaultMap(const char *filename)
 			   // Steam Windows
 			   StrEndsWith(buf, "WOLFENSTEIN 3D/BASE") ||
 			   StrEndsWith(buf, "SPEAR OF DESTINY/BASE") ||
+			   // GOG/Steam Windows
+			   StrEndsWith(buf, "Super 3-D Noah's Ark") ||
 			   // Steam Linux
 			   StrEndsWith(buf, "Wolfenstein 3D/base") ||
 			   StrEndsWith(buf, "Spear of Destiny/base"));
@@ -601,18 +634,19 @@ bail:
 
 static void LoadSounds(const SoundDevice *s, const CWolfMap *map);
 static void LoadMission(
-	CampaignSetting *c, const map_t tileClasses, const CWolfMap *map,
+	CampaignSetting *c, const map_t tileClasses, CWolfMap *map,
 	const int spearMission, const int missionIndex, const int numMissions);
 typedef struct
 {
-	const CWolfMap *Map;
+	CWolfMap *Map;
 	MusicType Type;
 } CampaignSongData;
-static Mix_Chunk *GetCampaignSong(void *data)
+static bool GetCampaignSong(MusicChunk *chunk, void *data)
 {
 	CampaignSongData *csd = data;
 	const int songIndex = songsCampaign[csd->Type];
-	return LoadMusic(csd->Map, CWAudioGetSong(csd->Map->type, songIndex));
+	return LoadMusic(
+		csd->Map, chunk, CWAudioGetSong(csd->Map->type, songIndex));
 }
 int MapWolfLoad(
 	const char *filename, const int spearMission, CampaignSetting *c)
@@ -646,7 +680,8 @@ int MapWolfLoad(
 		csd->Type = i;
 		c->CustomSongs[i].Data = csd;
 		c->CustomSongs[i].GetData = GetCampaignSong;
-		c->CustomSongs[i].Chunk = NULL;
+		c->CustomSongs[i].isMusic = false;
+		c->CustomSongs[i].u.Chunk = NULL;
 	}
 
 	char buf[CDOGS_PATH_MAX];
@@ -713,6 +748,10 @@ int MapWolfLoad(
 		MapNewScanArchive(filename, NULL, &numMissions);
 	}
 
+	CFREE(c->Description);
+	const char *description = CWGetDescription(map, spearMission);
+	CSTRDUP(c->Description, description);
+
 	CharacterStoreCopy(&c->characters, &cs, &gPlayerTemplates.CustomClasses);
 
 	for (int i = 0; i < map->nLevels; i++)
@@ -806,8 +845,8 @@ static Mix_Chunk *LoadSoundData(const CWolfMap *map, const int i)
 	}
 	SDL_AudioCVT cvt;
 	SDL_BuildAudioCVT(
-		&cvt, AUDIO_U8, 1, SND_RATE, CDOGS_SND_FMT, CDOGS_SND_CHANNELS,
-		CDOGS_SND_RATE);
+		&cvt, AUDIO_U8, 1, CWGetAudioSampleRate(map), CDOGS_SND_FMT,
+		CDOGS_SND_CHANNELS, CDOGS_SND_RATE);
 	cvt.len = (int)len;
 	cvt.buf = (Uint8 *)SDL_malloc(cvt.len * cvt.len_mult);
 	memcpy(cvt.buf, data, len);
@@ -877,17 +916,18 @@ static void LoadEntity(
 
 typedef struct
 {
-	const CWolfMap *Map;
+	CWolfMap *Map;
 	int MissionIndex;
 } MissionSongData;
-static Mix_Chunk *GetMissionSong(void *data)
+static bool GetMissionSong(MusicChunk *chunk, void *data)
 {
 	MissionSongData *msd = data;
 	return LoadMusic(
-		msd->Map, CWAudioGetLevelMusic(msd->Map->type, msd->MissionIndex));
+		msd->Map, chunk,
+		CWAudioGetLevelMusic(msd->Map->type, msd->MissionIndex));
 }
 static void LoadMission(
-	CampaignSetting *c, const map_t tileClasses, const CWolfMap *map,
+	CampaignSetting *c, const map_t tileClasses, CWolfMap *map,
 	const int spearMission, const int missionIndex, const int numMissions)
 {
 	const CWLevel *level = &map->levels[missionIndex];
@@ -931,7 +971,8 @@ static void LoadMission(
 		msd->MissionIndex = missionIndex;
 		m.Music.Data.Chunk.Data = msd;
 		m.Music.Data.Chunk.GetData = GetMissionSong;
-		m.Music.Data.Chunk.Chunk = NULL;
+		m.Music.Data.Chunk.isMusic = false;
+		m.Music.Data.Chunk.u.Chunk = NULL;
 
 		MissionStaticInit(&m.u.Static);
 
@@ -1038,6 +1079,9 @@ static void LoadTile(
 		break;
 	case CWTILE_AREA:
 		break;
+	case CWTILE_BACKGROUND:
+		// TODO: implement
+		break;
 	case CWTILE_SECRET_EXIT: {
 		// Secret exits need an elevator tile on the left or right
 		const struct vec2i vLeft = svec2i(v.x - 1, v.y);
@@ -1828,7 +1872,9 @@ typedef enum
 	CHAR_UBERMUTANT,
 	CHAR_DEATH_KNIGHT,
 	CHAR_GHOST,
-	CHAR_ANGEL
+	CHAR_ANGEL,
+	CHAR_KANGAROO,
+	CHAR_ELEPHANT
 } WolfChar;
 
 static bool MakeWallWalkable(Mission *m, const struct vec2i v);
@@ -2226,6 +2272,12 @@ static void LoadEntity(
 		CArrayPushBack(&m->u.Static.Exits, &e);
 	}
 	break;
+	case CWENT_NEXT_LEVEL:
+		// TODO: implement
+		break;
+	case CWENT_SECRET_LEVEL:
+		// TODO: implement
+		break;
 	case CWENT_GHOST:
 		LoadChar(m, v, DIRECTION_DOWN, (int)CHAR_GHOST, false, bossObjIdx);
 		break;
@@ -2416,6 +2468,16 @@ static void LoadEntity(
 			m, v, DIRECTION_DOWN, (int)CHAR_PACMAN_GHOST_BLUE, false,
 			bossObjIdx);
 		break;
+	case CWENT_KERRY_KANGAROO:
+		// TODO: implement
+		LoadChar(m, v, DIRECTION_DOWN, (int)CHAR_KANGAROO, false,
+			bossObjIdx);
+		break;
+	case CWENT_ERNIE_ELEPHANT:
+		// TODO: implement
+		LoadChar(m, v, DIRECTION_DOWN, (int)CHAR_ELEPHANT, false,
+			bossObjIdx);
+		break;
 	default:
 		break;
 	}
@@ -2510,6 +2572,14 @@ void MapWolfLoadCampaignsFromSystem(CampaignList *list)
 	{
 		TryLoadSpearGOG(list, numLoaded);
 	}
+
+	// Super 3D Noah's Ark
+	fsg_get_steam_game_path(buf, N3D_STEAM_NAME);
+	if (!TryLoadCampaign(list, buf))
+	{
+		fsg_get_gog_game_path(buf, N3D_GOG_ID);
+		TryLoadCampaign(list, buf);
+	}
 }
 static bool TryLoadSpearSteamVanilla(CampaignList *list);
 static bool TryLoadSpearSteamInWolf3D(CampaignList *list);
