@@ -1,7 +1,7 @@
 /*
 	C-Dogs SDL
 	A port of the legendary (and fun) action/arcade cdogs.
-	Copyright (c) 2013-2021 Cong Xu
+	Copyright (c) 2013-2023 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -36,11 +36,12 @@
 
 WeaponClasses gWeaponClasses;
 
-static const char *GunTypeStr(const GunType t)
+const char *GunTypeStr(const GunType t)
 {
 	switch (t)
 	{
 		T2S(GUNTYPE_NORMAL, "Normal");
+		T2S(GUNTYPE_MELEE, "Melee");
 		T2S(GUNTYPE_GRENADE, "Grenade");
 		T2S(GUNTYPE_MULTI, "Multi");
 	default:
@@ -48,9 +49,26 @@ static const char *GunTypeStr(const GunType t)
 	}
 }
 
+void GunTypeGetSlotStartEnd(const GunType gt, int *start, int *end)
+{
+	switch (gt)
+	{
+	case GUNTYPE_MELEE:
+		*start = *end = MELEE_SLOT;
+		break;
+	case GUNTYPE_GRENADE:
+		*start = *end = GRENADE_SLOT;
+		break;
+	default:
+		*start = 0;
+		*end = 1;
+		break;
+	}
+}
+
 // Initialise all the static weapon data
 #define VERSION 3
-void WeaponClassesInitialize(WeaponClasses *wcs)
+	void WeaponClassesInitialize(WeaponClasses *wcs)
 {
 	memset(wcs, 0, sizeof *wcs);
 	CArrayInit(&wcs->Guns, sizeof(WeaponClass));
@@ -125,12 +143,18 @@ static void LoadWeaponClass(WeaponClass *wc, json_t *node, const int version)
 	}
 	else
 	{
+		bool canShoot = true;
+		LoadBool(&canShoot, node, "CanShoot");
 		bool isGrenade = false;
 		LoadBool(&isGrenade, node, "IsGrenade");
 		if (isGrenade)
 		{
 			wc->Type = GUNTYPE_GRENADE;
 		}
+		else if (!canShoot)
+		{
+			wc->Type = GUNTYPE_MELEE;
+		}
 	}
 
 	char *tmp;
@@ -148,7 +172,6 @@ static void LoadWeaponClass(WeaponClass *wc, json_t *node, const int version)
 		wc->u.Normal.MuzzleFlash =
 			StrParticleClass(&gParticleClasses, "muzzle_flash_default");
 		wc->u.Normal.AmmoId = -1;
-		wc->u.Normal.CanShoot = true;
 		wc->CanDrop = true;
 	}
 
@@ -175,6 +198,14 @@ static void LoadWeaponClass(WeaponClass *wc, json_t *node, const int version)
 		wc->Description = tmp;
 	}
 
+	tmp = NULL;
+	LoadStr(&tmp, node, "Prerequisite");
+	if (tmp != NULL)
+	{
+		CFREE(wc->Prerequisite);
+		wc->Prerequisite = tmp;
+	}
+
 	LoadInt(&wc->Lock, node, "Lock");
 
 	LoadSoundFromNode(&wc->SwitchSound, node, "SwitchSound");
@@ -183,10 +214,16 @@ static void LoadWeaponClass(WeaponClass *wc, json_t *node, const int version)
 
 	LoadStr(&wc->DropGun, node, "DropGun");
 
-	switch (wc->Type)
+	if (wc->Type == GUNTYPE_MULTI)
 	{
-	case GUNTYPE_NORMAL:
-	case GUNTYPE_GRENADE: // fallthrough
+		int i = 0;
+		for (const json_t *gunNode = gunsNode->child->child; gunNode;
+			 gunNode = gunNode->next, i++)
+		{
+			wc->u.Guns[i] = json_unescape(gunNode->text);
+		}
+	}
+	else
 	{
 		tmp = NULL;
 		LoadStr(&tmp, node, "Pic");
@@ -259,11 +296,10 @@ static void LoadWeaponClass(WeaponClass *wc, json_t *node, const int version)
 		LoadFloat(&wc->u.Normal.Spread.Width, node, "SpreadWidth");
 		LoadFloat(&wc->u.Normal.AngleOffset, node, "AngleOffset");
 
-		int muzzleHeight = 0;
-		LoadInt(&muzzleHeight, node, "MuzzleHeight");
-		if (muzzleHeight)
+		if (json_find_first_label(node, "MuzzleHeight"))
 		{
-			wc->u.Normal.MuzzleHeight = muzzleHeight * Z_FACTOR;
+			LoadInt(&wc->u.Normal.MuzzleHeight, node, "MuzzleHeight");
+			wc->u.Normal.MuzzleHeight *= Z_FACTOR;
 		}
 		if (json_find_first_label(node, "Elevation"))
 		{
@@ -293,8 +329,6 @@ static void LoadWeaponClass(WeaponClass *wc, json_t *node, const int version)
 			CFREE(tmp);
 		}
 
-		LoadBool(&wc->u.Normal.CanShoot, node, "CanShoot");
-
 		if (version < 3)
 		{
 			LoadInt(&wc->u.Normal.Shake.Amount, node, "ShakeAmount");
@@ -307,20 +341,8 @@ static void LoadWeaponClass(WeaponClass *wc, json_t *node, const int version)
 				&wc->u.Normal.Shake.CameraSubjectOnly, shake,
 				"CameraSubjectOnly");
 		}
-	}
-	break;
-	case GUNTYPE_MULTI: {
-		int i = 0;
-		for (const json_t *gunNode = gunsNode->child->child; gunNode;
-			 gunNode = gunNode->next, i++)
-		{
-			wc->u.Guns[i] = json_unescape(gunNode->text);
-		}
-	}
-	break;
-	default:
-		CASSERT(false, "unknown gun type");
-		break;
+
+		LoadBool(&wc->u.Normal.Auto, node, "Auto");
 	}
 
 	wc->IsRealGun = true;
@@ -328,19 +350,25 @@ static void LoadWeaponClass(WeaponClass *wc, json_t *node, const int version)
 	if (version < 2)
 	{
 		CASSERT(wc->Type != GUNTYPE_MULTI, "unexpected gun type");
-		if (!wc->u.Normal.CanShoot)
+		if (wc->Type == GUNTYPE_MELEE)
 		{
 			wc->Lock = 0;
 		}
 	}
 
+	LoadInt(&wc->Price, node, "Price");
+
 	LOG(LM_MAP, LL_DEBUG, "loaded %s name(%s) lock(%d)...",
 		GunTypeStr(wc->Type), wc->name, wc->Lock);
 	LOG(LM_MAP, LL_DEBUG, "...canDrop(%s)", wc->CanDrop ? "true" : "false");
-	switch (wc->Type)
+	if (wc->Type == GUNTYPE_MULTI)
+	{
+		LOG(LM_MAP, LL_DEBUG, "...guns{%s, %s}",
+			wc->u.Guns[0] ? wc->u.Guns[0] : "",
+			wc->u.Guns[1] ? wc->u.Guns[1] : "");
+	}
+	else
 	{
-	case GUNTYPE_NORMAL:
-	case GUNTYPE_GRENADE: // fallthrough
 		LOG(LM_MAP, LL_DEBUG, "bullets(");
 		CA_FOREACH(const BulletClass *, bc, wc->u.Normal.Bullets)
 		if (_ca_index > 0)
@@ -360,25 +388,19 @@ static void LoadWeaponClass(WeaponClass *wc, json_t *node, const int version)
 			wc->u.Normal.Spread.Width, wc->u.Normal.Spread.Count,
 			wc->u.Normal.AngleOffset, wc->u.Normal.MuzzleHeight);
 		LOG(LM_MAP, LL_DEBUG,
-			"...elevation(%d-%d) muzzleFlash(%s) brass(%s) canShoot(%s)...",
+			"...elevation(%d-%d) muzzleFlash(%s) brass(%s)...",
 			wc->u.Normal.ElevationLow, wc->u.Normal.ElevationHigh,
 			wc->u.Normal.MuzzleFlash != NULL ? wc->u.Normal.MuzzleFlash->Name
 											 : "",
-			wc->u.Normal.Brass != NULL ? wc->u.Normal.Brass->Name : "",
-			wc->u.Normal.CanShoot ? "true" : "false");
+			wc->u.Normal.Brass != NULL ? wc->u.Normal.Brass->Name : "");
 		LOG(LM_MAP, LL_DEBUG, "...shake{amount(%d), cameraSubjectOnly(%s)}",
 			wc->u.Normal.Shake.Amount,
 			wc->u.Normal.Shake.CameraSubjectOnly ? "true" : "false");
-		break;
-	case GUNTYPE_MULTI:
-		LOG(LM_MAP, LL_DEBUG, "...guns{%s, %s}",
-			wc->u.Guns[0] ? wc->u.Guns[0] : "",
-			wc->u.Guns[1] ? wc->u.Guns[1] : "");
-		break;
-	default:
-		CASSERT(false, "unknown gun type");
-		break;
+		LOG(LM_MAP, LL_DEBUG, "...auto(%s)",
+			wc->u.Normal.Auto ? "true" : "false");
 	}
+
+	LOG(LM_MAP, LL_DEBUG, "price(%d)", wc->Price);
 }
 void WeaponClassesTerminate(WeaponClasses *wcs)
 {
@@ -398,23 +420,19 @@ static void WeaponClassTerminate(WeaponClass *wc)
 {
 	CFREE(wc->name);
 	CFREE(wc->Description);
+	CFREE(wc->Prerequisite);
 	CFREE(wc->DropGun);
-	switch (wc->Type)
+	if (wc->Type == GUNTYPE_MULTI)
 	{
-	case GUNTYPE_NORMAL:
-	case GUNTYPE_GRENADE: // fallthrough
-		CFREE(wc->u.Normal.Sprites);
-		CArrayTerminate(&wc->u.Normal.Bullets);
-		break;
-	case GUNTYPE_MULTI:
 		for (int i = 0; i < MAX_BARRELS; i++)
 		{
 			CFREE(wc->u.Guns[i]);
 		}
-		break;
-	default:
-		CASSERT(false, "unknown gun type");
-		break;
+	}
+	else
+	{
+		CFREE(wc->u.Normal.Sprites);
+		CArrayTerminate(&wc->u.Normal.Bullets);
 	}
 	memset(wc, 0, sizeof *wc);
 }
@@ -576,8 +594,7 @@ float WeaponClassGetMuzzleHeight(
 
 bool WeaponClassHasMuzzle(const WeaponClass *wc)
 {
-	return wc->Type == GUNTYPE_NORMAL && wc->u.Normal.Sprites != NULL &&
-		   wc->u.Normal.CanShoot;
+	return wc->Type == GUNTYPE_NORMAL && wc->u.Normal.Sprites != NULL;
 }
 bool WeaponClassIsHighDPS(const WeaponClass *wc)
 {
@@ -634,10 +651,14 @@ bool WeaponClassCanShoot(const WeaponClass *wc)
 		return WeaponClassCanShoot(WeaponClassGetBarrel(wc, 0)) ||
 			   WeaponClassCanShoot(WeaponClassGetBarrel(wc, 1));
 	}
-	return wc->u.Normal.CanShoot;
+	return wc->Type != GUNTYPE_MELEE;
 }
 int WeaponClassNumBarrels(const WeaponClass *wc)
 {
+	if (wc == NULL)
+	{
+		return 0;
+	}
 	return wc->Type == GUNTYPE_MULTI ? 2 : 1;
 }
 const WeaponClass *WeaponClassGetBarrel(
@@ -663,6 +684,42 @@ const BulletClass *WeaponClassGetBullet(
 	CA_FOREACH_END()
 	return NULL;
 }
+const WeaponClass *WeaponClassGetPrerequisite(const WeaponClass *wc)
+{
+	if (wc == NULL || wc->Prerequisite == NULL)
+	{
+		return NULL;
+	}
+	return StrWeaponClass(wc->Prerequisite);
+}
+static const WeaponClass *FindAncestor(const WeaponClass *wc)
+{
+	for (;;)
+	{
+		const WeaponClass *parent = WeaponClassGetPrerequisite(wc);
+		if (parent == NULL)
+		{
+			return wc;
+		}
+		wc = parent;
+	}
+}
+bool WeaponClassesAreRelated(const WeaponClass *wc1, const WeaponClass *wc2)
+{
+	// Find the ancestors of both, and check if they match
+	return wc1 != NULL && wc2 != NULL &&
+		   FindAncestor(wc1) == FindAncestor(wc2);
+}
+
+int WeaponClassFullPrice(const WeaponClass *wc)
+{
+	// Sum of prices for this weapon and all its prerequisites
+	if (wc == NULL)
+	{
+		return 0;
+	}
+	return wc->Price + WeaponClassFullPrice(WeaponClassGetPrerequisite(wc));
+}
 
 void BulletAndWeaponInitialize(
 	BulletClasses *b, WeaponClasses *wcs, const char *bpath, const char *gpath)
