@@ -2,7 +2,7 @@
 	C-Dogs SDL
 	A port of the legendary (and fun) action/arcade cdogs.
 
-	Copyright (c) 2013-2015, 2018, 2020 Cong Xu
+	Copyright (c) 2013-2015, 2018, 2020, 2022-2023 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -38,29 +38,37 @@ typedef enum
 	WEAPON_MENU_CANCEL
 } WeaponMenuResult;
 typedef struct
-{
-	MenuDisplayPlayerData display;
-	int PlayerUID;
-	int EquipSlot;
-	const WeaponClass *SelectedGun;
-	WeaponMenuResult SelectResult;
-} WeaponMenuData;
-typedef struct
 {
 	MenuSystem ms;
-	MenuSystem msEquip;
-	menu_t *gunMenu;
-	menu_t *grenadeMenu;
-	bool equipping;
-	WeaponMenuData data;
+	bool Active;
+	int PlayerUID;
+	WeaponMenuResult SelectResult;
+	int slot;
+	const NamedSprites *menuBGSprites;
+	int idx;
+	CArray weaponIndices;	   // of int
+	const CArray *weapons;	   // of const WeaponClass *
+	const CArray *weaponIsNew; // of bool
+	CArray weaponMeta;  // of DrawGunMeta
+	struct vec2i size;
+	int cols;
+	int scroll;
 } WeaponMenu;
 
 void WeaponMenuCreate(
-	WeaponMenu *menu, const CArray *weapons, const int numPlayers,
-	const int player, const int playerUID, EventHandlers *handlers,
+	WeaponMenu *menu, const CArray *weapons, const CArray *weaponIsNew,
+	const int playerUID, const int slot, const struct vec2i pos,
+	const struct vec2i size, EventHandlers *handlers,
 	GraphicsDevice *graphics);
 void WeaponMenuTerminate(WeaponMenu *menu);
 
+void WeaponMenuReset(WeaponMenu *menu);
+void WeaponMenuActivate(WeaponMenu *menu);
 void WeaponMenuUpdate(WeaponMenu *menu, const int cmd);
-bool WeaponMenuIsDone(const WeaponMenu *menu);
 void WeaponMenuDraw(const WeaponMenu *menu);
+
+int WeaponMenuSelectedCostDiff(const WeaponMenu *menu);
+
+void DrawWeaponAmmo(
+	GraphicsDevice *g, const PlayerData *p, const WeaponClass *wc,
+	const color_t mask, const struct vec2i pos, const struct vec2i slotSize);
